from .MrJattParser import MrJattParser

class MusicWebsiteFactory:

	def getParser(self,parser_name):
		if parser_name.lower()=="mr jatt":
			return MrJattParser()
		else:
			raise NotImplementedError('parser for this music website not implemented')	


# skele/commands/download.py
"""The hello command."""
 
 
from json import dumps
 
from .base import Base
from .FactoryProducer import FactoryProducer
from .FileDownload import FileDownload
class Download(Base):
    """Say hello, world!"""
 
    def run(self):
        #print 'Hello, world!'
        #print 'You supplied the following options:', dumps(self.options, indent=2, sort_keys=True)
        name=self.options["TEXT"]
        '''p=Parse(name)
       	download_url=p.get_download_url()
    	file_url=p.get_file_download_url(download_url)
    	p.file_download_using_wget(file_url)
    	'''
        factory = FactoryProducer()
    	p = factory.getFactory("search engine parser factory")
    	p=p.getParser("google")
    	website_url = p.Parse(name,"mr jatt")
    	q = factory.getFactory("music website parser factory")
    	q = q.getParser("mr jatt")
    	download_url = q.Parse(website_url,name)
        file_download=FileDownload()
    	file_download.file_download_cross_platform(download_url)




from MusicWebsiteParser.MusicWebsiteFactory import MusicWebsiteFactory
from SearchEngineParser.SearchEngineFactory import SearchEngineFactory

class FactoryProducer:

	 def getFactory(self,factory_name):
	 	if factory_name.lower()=="search engine parser factory":
	 		return SearchEngineFactory()
	 	elif factory_name.lower()=="music website parser factory":
	 		return MusicWebsiteFactory()
	 	else:
	 		raise NotImplementedError("%s not implemented"%factory_name)		
	


class SearchEngineParser:
	'''It is the base class for Search Engine Parser
	Every Search Engine Parser needs to implement this class
	'''
	def Parse(self,song_name,website):
		'''
		It will search for "song_name website"  in the Search Engine results 
		and return the url of website
		'''
		raise NotImplementedError('You must implement the Parse() method yourself!')


"""
song
 
Usage:
  song -d TEXT...
  song -h | --help
  song --version
 
Options:
  -h --help                         Show this screen.
  --version                         Show version.
 
Examples:
  song -d wo lamhe
 
Help:
  For help using this tool, please open an issue on the Github repository:
  https://github.com/ankitmathur3193/song-cli
"""
from inspect import getmembers, isclass

from docopt import docopt

from . import __version__ as VERSION

def main():
    """Main CLI entrypoint."""
    #print VERSION
    from commands.download import Download
    options = docopt(__doc__, version=VERSION)
    #print "You reached here"
    #print options
    
    p=Download(options)
    p.run()


from .MusicWebsiteParser import MusicWebsiteParser
from ..FileDownload import FileDownload
from BeautifulSoup import BeautifulSoup
import re

class MrJattParser(MusicWebsiteParser):
	'''
	Parser for https://mr-jatt.com/

	'''
	

	def missing_schema(self,html,song_name):
		'''
		It will print the list of songs that can be downloaded
		'''
		#html=self.get_html_response(url)
		soup=BeautifulSoup(html)
		name=' '.join(song_name)
		print '%s not found'%name
		print "But you can download any of the following songs :"
		a_list=soup.findAll('a','touch')
		for x in xrange(len(a_list)-1):
			r=a_list[x]
			p=str(r)
			q=re.sub(r'<a.*/>|<span.*">|</span>|</a>','',p)
			print q


	def Parse(self,url,song_name):
		'''
		It will the resource URL if song is found,
		Otherwise it will return the list of songs that can be downloaded
		'''
		file_download=FileDownload()
		html=file_download.get_html_response(url)
		soup=BeautifulSoup(html)
		a_list=soup.findAll('a','touch')
		#print a_list
		text=[str(x) for x in a_list]
		text=''.join(text)
		text=text.lower()
		string1='download in 48 kbps'
		string2='download in 128 kbps'
		string3='download in 320 kbps'

		href=''
		if string3 in text:
			print 'Downloading in 320 kbps'
			href=a_list[2].get('href')
			
		elif string2 in text:
			print 'Downloading in 128 kbps'
			href=a_list[1].get('href')
			
		elif string1 in text:
			print 'Downloading in 48 kbps'	
			href=a_list[0].get('href')
		else:
			self.missing_schema(html,song_name)
			quit()		

		return href 
			




from .GoogleParser import GoogleParser

class SearchEngineFactory:

	def getParser(self,parser_name):
		if parser_name.lower()=="google":
			return GoogleParser()
		else:
			raise NotImplementedError("parser for this search engine is not implemented yet")	


class MusicWebsiteParser:
	'''
	Base class for music website parsers
	Every parser needs to implement it

	'''
	def Parse(self,url):
		'''It will parse the html page specified by url
		and return the resource url from where the music file needs to be downloaded
		'''
		raise NotImplementedError('You must implement the Parse() method yourself!')


from codecs import open
from os.path import abspath, dirname, join
from subprocess import call

from setuptools import Command, find_packages, setup
from song import __version__ as VERSION
this_dir = abspath(dirname(__file__))
with open(join(this_dir, 'README.rst'), encoding='utf-8') as file:
    long_description = file.read()


setup(
	name = 'song',
	version = VERSION,
	author = 'ankit mathur',
	author_email = 'ankitmathur.dtu@gmail.com',
	license = 'UNLICENSE',
	description = ' A script to download hindi and punjabi songs from internet',
	long_description = long_description,
	url = 'https://github.com/ankitmathur3193/song-cli',
	keywords = 'song download script',
	packages = find_packages(exclude=['docs', 'tests*']),
	install_requires = ['docopt','requests','BeautifulSoup','tqdm'],
	classifiers = [
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
	entry_points = {
        'console_scripts': [
            'song=song.cli:main',
        ],
    },

	)	


from .SearchEngineParser import SearchEngineParser
from ..FileDownload import FileDownload
from BeautifulSoup import BeautifulSoup

class GoogleParser(SearchEngineParser):
	'''
	A parser for google search Engine
	'''

	def google_url(self,song_name,website):
		''' It will return the google url to be searched'''
		name='+'.join(song_name)
		prefix='https://www.google.co.in/search?q='	
		website=website.split(" ")
		suffix='+'.join(website)
		url=prefix+name+suffix
		#print url
		return url

	def parse_google(self,html):
		'''It will parse google html response
			and return the first url
		 '''
		soup = BeautifulSoup(html)
		href=soup.find('div','g').find('a').get('href')
		href_list=href.split('&')
		download_url=href_list[0]
		download_url=download_url.strip()
		download_url=download_url.replace('/url?q=','')
		return download_url


	def Parse(self,song_name,website):
		'''
		song_name is a list of strings
		website is a string
		It will return the url from where music file needs to be downloaded
		'''
		url_to_be_parsed=self.google_url(song_name,website)
		file_download=FileDownload()
		html=file_download.get_html_response(url_to_be_parsed)
		website_url=self.parse_google(html)
		return website_url






# skele/commands/base.py
"""The base command."""
 
 
class Base(object):
    """A base command."""
 
    def __init__(self, options, *args, **kwargs):
        self.options = options
        self.args = args
        self.kwargs = kwargs
 
    def run(self):
        raise NotImplementedError('You must implement the run() method yourself!')


import requests
import os
import os.path
import subprocess
from tqdm import tqdm

class FileDownload:

	def get_html_response(self,url):
		'''It will download the html page specified by url and return the html response '''
		print "Downloading page %s .."%url
		try:
			response=requests.get(url,timeout=50)
		except requests.exceptions.SSLError:
			try:
				response=requests.get(url,verify=False,timeout=50)
			except requests.exceptions.RequestException as e:
				print e
				quit()
		except requests.exceptions.RequestException as e:
			print e	
			quit()

		return response.content	


	def file_download_using_requests(self,url):
		'''It will download file specified by url using requests module'''
		file_name=url.split('/')[-1]

		if os.path.exists(os.path.join(os.getcwd(),file_name)):
			print 'File already exists'
			return
		#print 'Downloading file %s '%file_name
		#print 'Downloading from %s'%url

		

		try:
			r=requests.get(url,stream=True,timeout=200)
		except requests.exceptions.SSLError:
			try:
				response=requests.get(url,stream=True,verify=False,timeout=200)
			except requests.exceptions.RequestException as e:
				print e
				quit()		
		except requests.exceptions.RequestException as e:
				print e
				quit()	
		chunk_size = 1024
		total_size = int(r.headers['Content-Length'])
		total_chunks = total_size/chunk_size

		file_iterable = r.iter_content(chunk_size = chunk_size)
		tqdm_iter = tqdm(iterable = file_iterable,total = total_chunks,unit = 'KB',
			leave = False
			)		
		with open(file_name,'wb') as f:
			for data in tqdm_iter:
				f.write(data)

				
		#total_size=float(r.headers['Content-Length'])/(1024*1024)
		'''print 'Total size of file to be downloaded %.2f MB '%total_size
		total_downloaded_size=0.0
		with open(file_name,'wb') as f:
			for chunk in r.iter_content(chunk_size=1*1024*1024):
				if chunk:
					size_of_chunk=float(len(chunk))/(1024*1024)
					total_downloaded_size+=size_of_chunk
					print '{0:.0%} Downloaded'.format(total_downloaded_size/total_size)
					f.write(chunk)'''

		print 'Downloaded file %s '%file_name


	def file_download_using_wget(self,url):
		'''It will download file specified by url using wget utility of linux '''
		file_name=url.split('/')[-1]
		print 'Downloading file %s '%file_name
		command='wget -c --read-timeout=50 --tries=3 -q --show-progress --no-check-certificate '
		url='"'+url+'"'
		command=command+url
		os.system(command)			

	def file_download_cross_platform(self,url):
		file_name=url.split('/')[-1]
		if os.path.exists(os.path.join(os.getcwd(),file_name)):
			print 'File already exists'
			return
		print 'Downloading file %s '%file_name,'wb'
		#command='wget -c --read-timeout=50 --tries=3 -q --show-progress --no-check-certificate '
		try:
			#raise Exception('I know Python!')
			print 'file download using wget'
			subprocess.call(['wget','-c','--read-timeout=50','--tries=3','-q','--show-progress','--no-check-certificate',url])
		except:
			print 'file download using requests'
			self.file_download_using_requests(url);	


						
			